@@ -1,37 +1,62 @@
+"""
+Source for widget frame, including all widget creation functions
+"""
+
 from TKinterModernThemes.ThemeStyles import ThemeStyles
 from functools import partial as PARTIAL #for calling funcs with args
 from warnings import warn
 from tkinter import ttk
-from typing import List, Union, Tuple
+from typing import List, Union, Tuple, Literal
 import math
 import tkinter as tk
 
 toolbartype = None
 
 #region validation funcs
-def isFloat(x):
+def isFloat(x) -> bool:
+    """
+    Validation function used for a widget, returns true if x is float
+    """
     try:
         float(x)
         return True
     except ValueError:
         return False
 
-def isConstrainedFloat(lower, upper, x):
+def isConstrainedFloat(lower, upper, x) -> bool:
+    """
+    Validation function used for a widget, returns true if x is within range
+    :param lower: lower bound
+    :param upper: upper bound
+    :param x: value
+    """
     return isFloat(x) and lower <= float(x) <= upper
 
-def isMember(values, x):
+def isMember(values, x) -> bool:
+    """
+    Validation function used for a widget, returns true if x is in values
+    """
     return x in values
 
 def adjust(lower, inputvar, pbvar, _var, _indx, _mode):
+    """
+    Copies variable value, but subtracts value
+    """
     pbvar.set(inputvar.get() - lower)
 #endregion
 #region help funcs
 def partial(command, *args):
+    """
+    Creates a partial function, but returns None if command is None
+    """
     if command is not None:
         return PARTIAL(command, *args)
     return None
 
 def noneDict(*args):
+    """
+    Used for replacing None with {}
+    """
     l = []
     for arg in args:
         if arg is None:
@@ -44,6 +69,19 @@ def noneDict(*args):
 class Widget:
     def __init__(self, widget, name: str, row: int, col: int, rowspan: int, colspan: int, text="",
                  command=None, args=()):
+        """
+        Internal structure for storing widget information
+
+        :param widget: refrence to tk widget
+        :param name: friendly, printable name
+        :param row: from grid
+        :param col: from grid
+        :param rowspan: from grid
+        :param colspan: from grid
+        :param text: more text for printing
+        :param command: run when widget is interacted with
+        :param args: args for command
+        """
         self.widget = widget
         self.name = name
         self.row = row
@@ -65,10 +103,14 @@ class Widget:
             return self.name + '("' + self.text + '")' + self.commandstr
 
     def debugPrint(self, recursive=True):
+        """
+        Calls internal debug print for widget
+        """
         self.widget.debugPrint(recursive)
         
 class WidgetList:
     def __init__(self):
+        """Class for storing groups of widgets"""
         self.widgetlist = []
         
     def append(self, item: Widget):
@@ -84,10 +126,16 @@ class WidgetList:
 #endregion
 #region tabulate
 def centerText(text, longesttext):
+    """
+    Internal function for printing debug info, called by tabulate
+    """
     ldif = longesttext - len(text)
     return " " * math.floor(ldif / 2) + text + " " * math.ceil(ldif / 2)
 
 def printRow(grid, i, longesttext):
+    """
+    Internal function for printing debug info, called by tabulate
+    """
     s = '|'
     row = grid[i]
     for col in range(0, len(row)):
@@ -106,6 +154,9 @@ def printRow(grid, i, longesttext):
 
 
 def printSeperator(grid: List[List[Widget]], i, longesttext):
+    """
+    Internal function for printing debug info, called by tabulate
+    """
     s = "+"
     if i == 0:
         row = [None] * len(grid[0])
@@ -124,6 +175,9 @@ def printSeperator(grid: List[List[Widget]], i, longesttext):
     print(s)
 
 def tabulate(widgets):
+    """
+    Internal function for printing debug info
+    """
     maxrow = 0
     maxcol = 0
 
@@ -187,7 +241,7 @@ class Notebook:
             frame.debugPrint(recursive)
 
 class PanedWindow:
-    def __init__(self, master, name, orient = 'vertical', **widgetkwargs):
+    def __init__(self, master, name, orient: Literal["vertical", 'horiztonal'] = 'vertical', **widgetkwargs):
         """Creates a widget frame based panedwindow."""
         self.panedwindow = ttk.PanedWindow(master, orient=orient, **widgetkwargs)
         self.windows = []
@@ -510,7 +564,7 @@ class WidgetFrame:
 
     def Entry(self, textvariable: tk.Variable, row: int = None, col: int = None, padx=10, pady=10, sticky="nsew",
               validatecommand=None, validatecommandargs=(), validatecommandmode='%P', invalidcommand=None,
-              invalidcommandargs=(), validate='all', rowspan: int = 1, colspan: int = 1,
+              invalidcommandargs=(), validate=None, rowspan: int = 1, colspan: int = 1,
               widgetkwargs: dict = None, gridkwargs: dict = None):
         """
         Creates a ttk.Entry widget.
@@ -526,12 +580,19 @@ class WidgetFrame:
         :param validatecommandmode: Callback substitution code, defaults to 'new value'
         :param invalidcommand: Command to be called if text is invalid
         :param invalidcommandargs: Args to be passed to invalid command
-        :param validate: When the widget should validate, default is 'all'
+        :param validate: When the widget should validate, default is 'all' if command is supplied, none otherwise
         :param rowspan: Passed to grid
         :param colspan: Passed to grid
         :param widgetkwargs: Passed to widget creation
         :param gridkwargs: Passed to grid placement
         """
+
+        if validate is None:
+            if validatecommand is not None:
+                validate = 'all'
+            else:
+                validate = 'none'
+
         widgetkwargs, gridkwargs = noneDict(widgetkwargs, gridkwargs)
         validatefunc = self.master.register(partial(validatecommand, *validatecommandargs))
         invalidfunc = self.master.register(partial(invalidcommand, *invalidcommandargs))
@@ -545,7 +606,8 @@ class WidgetFrame:
 
     def NumericalSpinbox(self, lower: float, upper: float, increment: float, variable: tk.Variable,
                          validatecommand=isConstrainedFloat, validatecommandargs=(), validatecommandmode='%P',
-                         validate='focusout', row: int = None, col: int = None, padx=10, pady=10, sticky="nsew",
+                         validate:Literal['none', 'focus', 'focusin', 'focusout', 'key', 'all']='focusout',
+                         row: int = None, col: int = None, padx=10, pady=10, sticky="nsew",
                          rowspan: int = 1, colspan: int = 1, widgetkwargs: dict = None, gridkwargs: dict = None):
         """
         Creates a ttk.Spinbox designed for numbers.
@@ -580,7 +642,8 @@ class WidgetFrame:
         return widget
 
     def NonnumericalSpinbox(self, values: list, variable: tk.Variable, validatecommand=isMember,
-                            validatecommandargs=(), validatecommandmode='%P', validate='focusout',
+                            validatecommandargs=(), validatecommandmode='%P',
+                            validate:Literal['none', 'focus', 'focusin', 'focusout', 'key', 'all']='focusout',
                             row: int = None, col: int = None, padx=10, pady=10, sticky="nsew", wrap: bool = True,
                             rowspan: int = 1, colspan: int = 1, widgetkwargs: dict = None, gridkwargs: dict = None):
         """
@@ -609,16 +672,17 @@ class WidgetFrame:
         widget = ttk.Spinbox(self.master, textvariable=variable, validatecommand=(validatefunc, validatecommandmode),
                              validate=validate, values=values, wrap=wrap, **widgetkwargs)
         row, col = self.getRow(row, col, rowspan, colspan)
-        widget.grid(row=row, column=col, padx=padx, pady=pady, sticky=sticky)
+        widget.grid(row=row, column=col, padx=padx, pady=pady, columnspan=colspan, rowspan=rowspan, sticky=sticky,
+                    **gridkwargs)
         name = "Spinbox"
         if len(str(values)) < 25:
             name = "Spinbox(" + str(values)[1:-2] + ")"
         self.widgets.append(Widget(widget, name, row, col, rowspan, colspan))
         return widget
 
-    def Treeview(self, columnnames: list, columnwidths: list, height: int, data: dict, datasubfilename: str,
-                 datacolumnnames: list = None, anchor='w', newframe = True, row: int = None, col: int =0, padx=10,
-                 pady=10, sticky="nsew", rowspan: int = 1, colspan: int = 1,
+    def Treeview(self, columnnames: list, columnwidths: list, height: int, data: dict, subentryname: str,
+                 datacolumnnames: list = None, openkey = None, anchor='w', newframe = True, row: int = None,
+                 col: int =0, padx=10, pady=10, sticky="nsew", rowspan: int = 1, colspan: int = 1,
                  widgetkwargs: dict = None, gridkwargs: dict = None):
         """
         Creates a ttk.Treeview. Will load in data presented in a nested dict.
@@ -627,8 +691,9 @@ class WidgetFrame:
         :param columnwidths: Width of each column
         :param height: number of rows
         :param data: nested dict
-        :param datasubfilename: key in dict to go to next layer
+        :param subentryname: key in dict to go to next layer
         :param datacolumnnames: defaults to columnnames, a mapping of columnnames to the data file
+        :param openkey: bool key to set subentries open or closed
         :param anchor: text position
         :param newframe: creates a frame for the treeview and scrollbar, needed if master frame contains other widgets
         :param row: Passed to grid, defaults to +1 of last item in col
@@ -650,7 +715,8 @@ class WidgetFrame:
         row, col = self.getRow(row, col, rowspan, colspan)
         if newframe:
             widgetFrame = ttk.Frame(self.master)
-            widgetFrame.grid(row=row, column=col, padx=padx, pady=pady, sticky=sticky, **gridkwargs)
+            widgetFrame.grid(row=row, column=col, padx=padx, pady=pady, sticky=sticky, columnspan=colspan,
+                             rowspan=rowspan, **gridkwargs)
         else:
             widgetFrame = self.master
 
@@ -675,9 +741,12 @@ class WidgetFrame:
                 for value in datacolumnnames[1:]:
                     values.append(obj.get(value, ""))
                 widget.insert(p, index='end', iid=iid[0], text=obj[datacolumnnames[0]], values=values)
-                if obj.get(datasubfilename, []):
-                    widget.item(iid[0], open=True)  # Open parents
-                traverse(iid[0], obj.get('subfiles', []), iid)
+                if obj.get(subentryname, []):
+                    if openkey is None:
+                        widget.item(iid[0], open=True)  # Open parents
+                    else:
+                        widget.item(iid[0], open=obj[openkey])
+                traverse(iid[0], obj.get(subentryname, []), iid)
 
         traverse('', data, [0])
         self.widgets.append(Widget(widget, "Treeview", row, col, rowspan, colspan))
@@ -709,7 +778,8 @@ class WidgetFrame:
         if default is None:
             default = values[0]
         widget = ttk.OptionMenu(self.master, variable, default, command=partial(command,*args), *values, **widgetkwargs)
-        widget.grid(row=row, column=col, padx=padx, pady=pady, sticky=sticky, rowspan=rowspan, columnspan=colspan, **gridkwargs)
+        widget.grid(row=row, column=col, padx=padx, pady=pady, sticky=sticky, rowspan=rowspan, columnspan=colspan,
+                    **gridkwargs)
         name = "OptionMenu"
         if len(str(values)) < 25:
             name = "OptionMenu(" + str(values)[1:-2] + ")"
@@ -792,12 +862,14 @@ class WidgetFrame:
         widgetkwargs, gridkwargs = noneDict(widgetkwargs, gridkwargs)
         row, col = self.getRow(row, col, rowspan, colspan)
         widget = Notebook(self.master, name, **widgetkwargs)
-        widget.notebook.grid(row = row, column=col, padx=padx, pady=pady, sticky=sticky, **gridkwargs)
+        widget.notebook.grid(row = row, column=col, padx=padx, pady=pady, sticky=sticky,
+                             columnspan=colspan, rowspan=rowspan, **gridkwargs)
         self.widgets.append(Widget(widget, "Notebook", row, col, rowspan, colspan, name))
 
         return widget
 
-    def PanedWindow(self, name, orient = 'vertical', row: int = None, col: int = None, padx=10, pady=10, sticky="nsew",
+    def PanedWindow(self, name, orient: Literal["vertical", 'horiztonal'] = 'vertical', row: int = None,
+                    col: int = None, padx=10, pady=10, sticky="nsew",
                     rowspan: int = 1, colspan: int = 1, widgetkwargs: dict = None, gridkwargs: dict = None):
 
         """
@@ -896,9 +968,9 @@ class WidgetFrame:
 
         return widget
 
-    def Scale(self, lower: float, upper: float, variable: tk.Variable, row: int = None, col: int = None, padx=10,
-              pady=10, sticky="ew", rowspan: int = 1, colspan: int = 1, widgetkwargs: dict = None,
-              gridkwargs: dict = None):
+    def Scale(self, lower: float, upper: float, variable: Union[tk.IntVar, tk.DoubleVar], row: int = None,
+              col: int = None, padx=10, pady=10, sticky="ew", rowspan: int = 1, colspan: int = 1,
+              widgetkwargs: dict = None, gridkwargs: dict = None):
         """
         Creates a ttk.Scale (horizontal only, vertical is not supported with these themes)
 
@@ -922,14 +994,14 @@ class WidgetFrame:
         self.widgets.append(Widget(widget, "Scale" + str((lower, upper)), row, col, rowspan, colspan))
         return widget
 
-    def Progressbar(self, variable: tk.Variable, mode="determinate", lower=0, upper=100, row: int = None,
-                    col: int = None, padx=10, pady=10, sticky="ew", rowspan: int = 1, colspan: int = 1,
-                    widgetkwargs: dict = None, gridkwargs: dict = None):
+    def Progressbar(self, variable: tk.Variable, mode: Literal["determinate", "indeterminate"] ="determinate", lower=0,
+                    upper=100, row: int = None, col: int = None, padx=10, pady=10, sticky="ew", rowspan: int = 1,
+                    colspan: int = 1, widgetkwargs: dict = None, gridkwargs: dict = None):
         """
         Creates a ttk.Progressbar. If lower is not 0, it will link a second variable for processing
 
         :param variable: tk.Variable to control progressbar
-        :param mode: "determinate" for variable control, "indetermiante" for animation
+        :param mode: "determinate" for variable control, "indeterminate" for animation
         :param lower: min value (0 is highly recommended)
         :param upper: max value
         :param row: passed to grid
